"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.callCustomMethod = void 0;
const method_utils_1 = require("./method-utils");
const http_method_1 = require("./http-method");
const weda_api_1 = require("./weda-api");
async function callCustomMethod(options) {
    var _a, _b, _c, _d, _e;
    if (!method_utils_1.isMethodEnabled(options.methodName, options, options.forceCall)) {
        throw new Error(`method ${options.methodName} not found in datasource ${(_a = options.dsConfig) === null || _a === void 0 ? void 0 : _a.name}(${(_b = options.dsConfig) === null || _b === void 0 ? void 0 : _b.id}) or not enabled in this datasource`);
    }
    /** 非测试模式才进行权限校验 */
    if (!options.isTestMode) {
        const hasPermission = await hasMethodPermission((_c = options.dsConfig) === null || _c === void 0 ? void 0 : _c.id, options.methodName);
        if (!hasPermission)
            throw new Error(`no permission to invoke method ${options.methodName} in datasource ${(_d = options.dsConfig) === null || _d === void 0 ? void 0 : _d.title}(${(_e = options.dsConfig) === null || _e === void 0 ? void 0 : _e.name})`);
    }
    const methodCfg = method_utils_1.getMethodCfg(options.dsConfig, options.methodName);
    const isHttp = http_method_1.isHttpMethod(methodCfg);
    if (isHttp) {
        return http_method_1.callHttpMethod({
            methodName: options.methodName,
            dsConfig: options.dsConfig,
            params: options.params,
            methodCfg,
            isTestMode: options.isTestMode,
        }, options.context);
    }
    const method = options.handlers[options.methodName];
    const result = await method(options.params, options.context);
    return {
        code: 0,
        data: result,
    };
}
exports.callCustomMethod = callCustomMethod;
/**
 * 检查数据源自定义方法方法是否有权限
 * @param dataSourceId 数据源ID (非数据源name)
 * @param methodName 方法名
 * @returns boolean
 */
async function hasMethodPermission(dataSourceId, methodName) {
    if (!dataSourceId)
        throw new Error('datasource info not found!');
    const resourceId = `${dataSourceId}#${methodName}`;
    try {
        const result = await weda_api_1.fetchWeda({
            action: 'DescribeResourcesPermission',
            data: {
                ResourceType: 'dataSource',
                ResourceIdList: [resourceId],
            },
        }).then(res => res.Data);
        if (result === null || result === void 0 ? void 0 : result.length) {
            const item = result.find((n) => n.ResourceId === resourceId);
            return item === null || item === void 0 ? void 0 : item.IsAccess;
        }
        return false;
    }
    catch (error) {
        throw new Error(`failed to call \`DescribeResourcesPermission\`: ${error.message}`);
    }
}
