"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createApp = void 0;
const fn_context_1 = require("./fn-context");
const utils_1 = require("./utils");
const builtin_method_1 = require("./builtin-method");
const call_custom_method_1 = require("./call-custom-method");
function createApp(options) {
    return async function main(sfcParams, context) {
        var _a;
        try {
            const dsConfig = options.dsConfig || {};
            const fnContext = fn_context_1.getFnContext(Object.assign(Object.assign({}, options), { params: sfcParams, rawContext: context }));
            // if (!isMethodEnabled(sfcParams.methodName, options, sfcParams.forceCall)) {
            //   throw new Error(`method ${sfcParams.methodName} not found in datasource ${dsConfig.name}(${dsConfig.id})
            //  or not enabled in this datasource`);
            // }
            let fnParams = sfcParams.params || {};
            const isDataBase = dsConfig.type === 'database';
            // 是否为旧数据源调用
            // @ts-ignore
            const isLegacyDs = options.isLegacyDs || (dsConfig.config && !dsConfig.schema);
            if (utils_1.isObj(sfcParams, 'defaultParams')) {
                fnParams = utils_1.mergeDeep(sfcParams.defaultParams, fnParams);
            }
            // 内建数据源调用weda后台实现
            if (isDataBase) {
                const viewId = sfcParams.viewId || ((_a = dsConfig.schema) === null || _a === void 0 ? void 0 : _a['x-viewId']);
                if (builtin_method_1.isBuiltinMethod(sfcParams.methodName)) {
                    const builtinResult = await builtin_method_1.callBuiltinMethod(Object.assign(Object.assign({}, sfcParams), { params: fnParams, viewId, 
                        // @ts-ignore
                        dataSourceName: dsConfig.name, useLegacy: isLegacyDs }), fnContext);
                    return builtinResult;
                }
            }
            return call_custom_method_1.callCustomMethod({
                dsConfig: options.dsConfig,
                params: fnParams,
                methodName: sfcParams.methodName,
                isTestMode: options.isTestMode,
                context: fnContext,
                handlers: options.handlers,
                forceCall: sfcParams.forceCall,
            });
        }
        catch (e) {
            const { isTestMode } = options;
            if (e instanceof utils_1.TCBError) {
                return {
                    code: e.code || -1,
                    message: e.message,
                    original: isTestMode ? (e.stack || e.message) : undefined,
                };
            }
            return {
                code: -1,
                message: e.message,
                original: isTestMode ? (e.stack || e.message) : undefined,
            };
        }
    };
}
exports.createApp = createApp;
