"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createCallDataSource = void 0;
const utils_1 = require("../utils");
const builtin_method_1 = require("../builtin-method");
const call_function_1 = require("../call-function");
const call_custom_method_1 = require("../call-custom-method");
function createCallDataSource(options) {
    return async function callDataSource(params) {
        var _a;
        const dsBasic = getDsBasicInfo(params.dataSourceName, {
            // @ts-ignore 兼容某些地方未直接传 dsNameIdMap 的问题
            dsNameIdMap: options.dsNameIdMap || ((_a = options.dsConfig) === null || _a === void 0 ? void 0 : _a.dsNameIdMap),
            dsConfig: options.dsConfig,
            isPreview: options.isPreview,
        });
        // 不论是否为当前数据源的方法, 只要是内置方法即可直接调用微搭后台, 无需中转
        if (builtin_method_1.isBuiltinMethod(params.methodName)) {
            return builtin_method_1.callBuiltinMethod({
                dataSourceName: dsBasic.name,
                methodName: params.methodName,
                params: params.params,
            }).then(unwrapResult);
            ;
        }
        // 如果调用的是其他方法的自定义方法, 则需要用 callFunction 调用
        if (!dsBasic.isSelf) {
            const fnParams = {
                methodName: params.methodName,
                params: params.params,
                wedaTarget: utils_1.getConfig('wedaTarget'),
                userSource: utils_1.getUserSource(),
            };
            return call_function_1.callFunction({
                name: dsBasic.dsFullName,
                data: fnParams,
            }).then(res => unwrapResult(res, true));
        }
        if (options.isTestMode)
            throw new Error('currently can not call current datasource\'s custom methods in test mode');
        // 当前数据源的方法调用自身的其他自定义方法
        //    此时需注意递归调用的问题
        if (!params.context)
            throw new Error('context is required when using callDataSource');
        return call_custom_method_1.callCustomMethod({
            dsConfig: options.dsConfig,
            params: params.params,
            methodName: params.methodName,
            context: params.context,
            handlers: options.handlers,
        }).then(unwrapResult);
    };
}
exports.createCallDataSource = createCallDataSource;
/**
 * 解包结果
 * @param res 云开发 callFunction 返回的原始结果
 * @returns
 */
function unwrapResult(res, isTcbResponse) {
    let dsRes = res;
    // 云开发链路层发生错误
    if (isTcbResponse) {
        if (res.code)
            throw new Error(`[tcb error]: code ${res.code}, message ${res.message}, requestId ${res.requestId}`);
        dsRes = res.result;
    }
    // 数据源云函数内部处理错误
    if (!dsRes)
        throw new Error('[datasource error]: get empty response from datasource');
    if (dsRes.code)
        throw new Error(`[datasource error]: code ${dsRes.code}, message ${dsRes.message}`);
    return dsRes.data;
}
function getDsBasicInfo(dsName, options) {
    var _a;
    const isSelf = dsName === 'self' || ((_a = options.dsConfig) === null || _a === void 0 ? void 0 : _a.name) === dsName;
    if (isSelf) {
        if (!options.dsConfig)
            throw new Error('can not get current datasource basic info when calling callDataSource');
        return {
            isSelf,
            id: options.dsConfig.id,
            name: dsName,
            dsFullName: utils_1.getDsFullName(options.dsConfig, options.isPreview),
        };
    }
    if (!options.dsNameIdMap || !options.dsNameIdMap[dsName])
        throw new Error(`can not get ${dsName}'s id when calling callDataSource`);
    return {
        isSelf,
        id: options.dsNameIdMap[dsName],
        name: dsName,
        dsFullName: utils_1.getDsFullName({ id: options.dsNameIdMap[dsName], name: dsName }, options.isPreview),
    };
}
